@@ -5,7 +5,7 @@ from django.utils.translation import ugettext as _
 from django.core.exceptions import ImproperlyConfigured
 from django.contrib.contenttypes import generic
 from odnoklassniki_api.models import OdnoklassnikiManager, OdnoklassnikiPKModel
-from odnoklassniki_api.decorators import fetch_all, atomic
+from odnoklassniki_api.decorators import fetch_all, atomic, opt_generator
 from odnoklassniki_api.fields import JSONField
 import logging
 
@@ -40,7 +40,7 @@ class Group(OdnoklassnikiPKModel):
         verbose_name = _('Odnoklassniki group')
         verbose_name_plural = _('Odnoklassniki groups')
 
-#     resolve_screen_name_type = 'group'
+    resolve_screen_name_type = 'GROUP'
     methods_namespace = 'group'
     remote_pk_field = 'uid'
     slug_prefix = 'group'
@@ -80,24 +80,15 @@ class Group(OdnoklassnikiPKModel):
     def parse(self, response):
         # in entity of discussion
         if 'main_photo' in response:
-            if 'id' in response['main_photo']:
-                del response['main_photo']['id']
+            response['main_photo'].pop('id', None)
             response.update(response.pop('main_photo'))
 
         # pop avatar, because self.pic50x50 the same
-        response.pop('picAvatar', None)
+        if 'picAvatar' in response:
+            response['pic50x50'] = response.pop('picAvatar')
 
         super(Group, self).parse(response)
 
-    @atomic
-    def update_users(self, **kwargs):
-        if 'odnoklassniki_users' not in settings.INSTALLED_APPS:
-            raise ImproperlyConfigured("Application 'odnoklassniki_users' not in INSTALLED_APPS")
-
-        ids = Group.remote.get_members_ids(group=self)
-        self.users = User.remote.fetch(ids=ids)
-
-        return self.users.all()
 
 '''
 Fields, dependent on other applications
@@ -109,17 +100,35 @@ def get_improperly_configured_field(app_name, decorate_property=False):
         field = property(field)
     return field
 
+
 if 'odnoklassniki_users' in settings.INSTALLED_APPS:
     from odnoklassniki_users.models import User
     from m2m_history.fields import ManyToManyHistoryField
-    users = ManyToManyHistoryField(User)
+    users = ManyToManyHistoryField(User, cache=True)
+
+    @atomic
+#    @opt_generator
+    def update_users(self, **kwargs):
+        ids = self.__class__.remote.get_members_ids(group=self)
+        # TODO: remove time_from of first updating
+#        initial = self.users.count() == 0
+
+        self.users = User.remote.fetch(ids=ids)
+
+#         if initial:
+#             self.users.get_query_set_through().update(time_from=None)
+
+        return self.users.all()
 else:
     users = get_improperly_configured_field('odnoklassniki_users', True)
+    update_users = users
+
 
 if 'odnoklassniki_discussions' in settings.INSTALLED_APPS:
     from odnoklassniki_discussions.models import Discussion
     discussions = generic.GenericRelation(Discussion, content_type_field='owner_content_type', object_id_field='owner_id')
     discussions_count = models.PositiveIntegerField(null=True)
+
     def fetch_discussions(self, **kwargs):
         return Discussion.remote.fetch(group=self, **kwargs)
 else:
@@ -127,7 +136,25 @@ else:
     discussions_count = discussions
     fetch_discussions = discussions
 
+
+if 'odnoklassniki_photos' in settings.INSTALLED_APPS:
+    from odnoklassniki_photos.models import Album
+    albums = generic.GenericRelation(Album, content_type_field='owner_content_type', object_id_field='owner_id')
+    albums_count = models.PositiveIntegerField(null=True)
+
+    def fetch_albums(self, **kwargs):
+        return Album.remote.fetch(group=self, **kwargs)
+else:
+    albums = get_improperly_configured_field('odnoklassniki_photos', True)
+    albums_count = albums
+    fetch_albums = albums
+
+
 Group.add_to_class('users', users)
+Group.add_to_class('update_users', update_users)
 Group.add_to_class('discussions', discussions)
 Group.add_to_class('discussions_count', discussions_count)
-Group.add_to_class('fetch_discussions', fetch_discussions)
\ No newline at end of file
+Group.add_to_class('albums', albums)
+Group.add_to_class('albums_count', albums_count)
+Group.add_to_class('fetch_discussions', fetch_discussions)
+Group.add_to_class('fetch_albums', fetch_albums)
\ No newline at end of file
